#!/bin/bash

# Only root user can run this script.
if [ "`id -u`" != "0" ]; then
  echo "ERROR: this script can only be run by the root user."
  exit 1
fi

echo "
This script generates a new Drupal client account for the Asterisk server
based on the entered data.  The new client account directory will be created
in the same directory as the script, and file/folder permissions will be
correctly set.  Default input values appear in brackets.
"

BASE_DIR=`dirname $0`

read -p "User/group running the webserver process [apache]: " WEBSERVER_USER
read -p "User/group running the Asterisk process [asterisk]: " ASTERISK_USER
read -p "User name of the Drupal client (auto-generated by the client) [www_example_com]: " DRUPAL_CLIENT_USER
read -p "Password for the Drupal client (chosen by you) [secret]: " DRUPAL_CLIENT_PASSWORD

WEBSERVER_USER="${WEBSERVER_USER:=apache}"
ASTERISK_USER="${ASTERISK_USER:=asterisk}"
DRUPAL_CLIENT_USER="${DRUPAL_CLIENT_USER:=www_example_com}"
DRUPAL_CLIENT_PASSWORD="${DRUPAL_CLIENT_PASSWORD:=secret}"

echo
echo "webserver user: $WEBSERVER_USER"
echo "Asterisk user: $ASTERISK_USER"
echo "Drupal client user name: $DRUPAL_CLIENT_USER"
echo "Drupal client password: $DRUPAL_CLIENT_PASSWORD"
echo

id $WEBSERVER_USER &> /dev/null
if [ "$?" != "0" ]; then
  echo "ERROR: The webserver user '$WEBSERVER_USER' does not exist -- aborting."
  exit 1
fi

id $ASTERISK_USER &> /dev/null
if [ "$?" != "0" ]; then
  echo "ERROR: The Asterisk user '$ASTERISK_USER' does not exist -- aborting."
  exit 1
fi


# Create main directory.
DRUPAL_ACCOUNT_DIR=${BASE_DIR}/${DRUPAL_CLIENT_USER}
mkdir -p $DRUPAL_ACCOUNT_DIR

# Create subdirectories in main directory.
ACCOUNT_SUBDIRS="cache messages recordings"
for dir in $ACCOUNT_SUBDIRS
do
  mkdir -p ${DRUPAL_ACCOUNT_DIR}/${dir}
done

# Start out by making the whole account directory structure owned by the
# webserver user, group Asterisk user, no 'world' access.
chown -R ${WEBSERVER_USER}:${ASTERISK_USER} $DRUPAL_ACCOUNT_DIR
chmod -R 770 $DRUPAL_ACCOUNT_DIR

# The recordings directory needs to be owned by the Asterisk user, webserver
# user group with guid set.
RECORDINGS_DIR=${DRUPAL_ACCOUNT_DIR}/recordings
chown ${ASTERISK_USER}:${WEBSERVER_USER} $RECORDINGS_DIR
chmod g+s $RECORDINGS_DIR

# Write the configuration file to the main account directory, group webserver,
# no 'world' access.
CONF_FILE=${DRUPAL_ACCOUNT_DIR}/account.conf
echo "[general]
pass=${DRUPAL_CLIENT_PASSWORD}" > $CONF_FILE
chgrp $WEBSERVER_USER $CONF_FILE
chmod 640 $CONF_FILE

echo "New account directory created at ${DRUPAL_ACCOUNT_DIR}"

exit 0

